@@ -35,6 +35,7 @@
 
 #include <glib.h>
 #include <gdk/gdk.h>
+#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
 #include <libxfce4util/libxfce4util.h>
@@ -197,9 +198,9 @@ struct _ScreenSaver {
     GList *floaters;
     gint max_floater_count;
 
-    gboolean should_do_rotations: 1;
-    gboolean should_show_paths : 1;
-    gboolean draw_ops_pending : 1;
+    guint should_do_rotations: 1;
+    guint should_show_paths : 1;
+    guint draw_ops_pending : 1;
 };
 
 static Path *path_new (Point *start_point,
@@ -1141,6 +1142,11 @@ main (int   argc,
         return EX_SOFTWARE;
     }
 
+    if (!GDK_IS_X11_DISPLAY (gdk_display_get_default ())) {
+        g_warning ("Unsupported windowing environment");
+        return EX_SOFTWARE;
+    }
+
     if ((filenames == NULL) || (filenames[0] == NULL) ||
             (filenames[1] != NULL)) {
         g_printerr (_("You must specify one image.  See --help for usage "
