@@ -37,8 +37,6 @@
 #include "gs-debug.h"
 
 #define SLEEPTIMEOUT 100000
-static void     gs_grab_class_init (GSGrabClass *klass);
-static void     gs_grab_init       (GSGrab      *grab);
 static void     gs_grab_finalize   (GObject     *object);
 
 static gpointer grab_object = NULL;
