@@ -46,8 +46,6 @@
 #include "gs-theme-manager.h"
 #include "subprocs.h"
 
-static void gs_job_class_init (GSJobClass *klass);
-static void gs_job_init       (GSJob      *job);
 static void gs_job_finalize   (GObject    *object);
 
 typedef enum {
@@ -269,7 +267,6 @@ get_env_vars (GtkWidget *widget) {
     GPtrArray         *env;
     const gchar       *display_name;
     gchar             *str;
-    int                i;
     static const char *allowed_env_vars[] = {
         "PATH",
         "SESSION_MANAGER",
@@ -277,7 +274,8 @@ get_env_vars (GtkWidget *widget) {
         "XAUTHLOCALHOSTNAME",
         "LANG",
         "LANGUAGE",
-        "DBUS_SESSION_BUS_ADDRESS"
+        "DBUS_SESSION_BUS_ADDRESS",
+        "G_DEBUG"
     };
 
     env = g_ptr_array_new ();
@@ -288,7 +286,7 @@ get_env_vars (GtkWidget *widget) {
     g_ptr_array_add (env, g_strdup_printf ("HOME=%s",
                                            g_get_home_dir ()));
 
-    for (i = 0; i < G_N_ELEMENTS (allowed_env_vars); i++) {
+    for (guint i = 0; i < G_N_ELEMENTS (allowed_env_vars); i++) {
         const char *var;
         const char *val;
         var = allowed_env_vars[i];
@@ -323,8 +321,7 @@ spawn_on_widget (GtkWidget  *widget,
     GError      *error = NULL;
     int          standard_error;
     int          child_pid;
-    int          id;
-    int          i;
+    int          id = 0;
 
     if (command == NULL) {
         return FALSE;
@@ -348,10 +345,10 @@ spawn_on_widget (GtkWidget  *widget,
                                        &child_pid,
                                        NULL,
                                        NULL,
-                                       &standard_error,
+                                       watch_func != NULL ? &standard_error : NULL,
                                        &error);
 
-    for (i = 0; i < env->len; i++) {
+    for (guint i = 0; i < env->len; i++) {
         g_free (g_ptr_array_index (env, i));
     }
     g_ptr_array_free (env, TRUE);
@@ -373,21 +370,22 @@ spawn_on_widget (GtkWidget  *widget,
         g_spawn_close_pid (child_pid);
     }
 
-    channel = g_io_channel_unix_new (standard_error);
-    g_io_channel_set_close_on_unref (channel, TRUE);
-    g_io_channel_set_flags (channel,
-                            g_io_channel_get_flags (channel) | G_IO_FLAG_NONBLOCK,
-                            NULL);
-    id = g_io_add_watch (channel,
-                         G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL,
-                         watch_func,
-                         user_data);
+    if (watch_func != NULL) {
+        channel = g_io_channel_unix_new (standard_error);
+        g_io_channel_set_close_on_unref (channel, TRUE);
+        g_io_channel_set_flags (channel,
+                                g_io_channel_get_flags (channel) | G_IO_FLAG_NONBLOCK,
+                                NULL);
+        id = g_io_add_watch (channel,
+                             G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL,
+                             watch_func,
+                             user_data);
+        g_io_channel_unref (channel);
+    }
     if (watch_id != NULL) {
         *watch_id = id;
     }
 
-    g_io_channel_unref (channel);
-
     return result;
 }
 
@@ -445,7 +443,7 @@ gs_job_is_running (GSJob *job) {
 
 gboolean
 gs_job_start (GSJob *job) {
-    gboolean result;
+    gboolean redirect_stderr, result;
 
     g_return_val_if_fail (job != NULL, FALSE);
     g_return_val_if_fail (GS_IS_JOB (job), FALSE);
@@ -469,10 +467,15 @@ gs_job_start (GSJob *job) {
         return FALSE;
     }
 
+    /* do not redirect stderr for our own commands */
+    redirect_stderr = ! g_str_has_prefix (job->priv->command, LIBEXECDIR "/xfce4-screensaver/floaters")
+                      && ! g_str_has_prefix (job->priv->command, LIBEXECDIR "/xfce4-screensaver/popsquares")
+                      && ! g_str_has_prefix (job->priv->command, LIBEXECDIR "/xfce4-screensaver/slideshow");
+
     result = spawn_on_widget (job->priv->widget,
                               job->priv->command,
                               &job->priv->pid,
-                              (GIOFunc)command_watch,
+                              redirect_stderr ? (GIOFunc)command_watch : NULL,
                               job,
                               &job->priv->watch_id);
 
