@@ -65,7 +65,8 @@
 #define LOGGED_IN_EMBLEM_SIZE 20
 #define LOGGED_IN_EMBLEM_ICON "emblem-default"
 
-static void gs_lock_plug_finalize   (GObject         *object);
+static void gs_lock_plug_constructed   (GObject         *object);
+static void gs_lock_plug_finalize      (GObject         *object);
 
 struct GSLockPlugPrivate {
     GtkWidget   *vbox;
@@ -178,12 +179,15 @@ process_is_running (const char * name) {
     int    num_processes;
     gchar *command = g_strdup_printf ("pidof %s | wc -l", name);
     FILE  *fp = popen(command, "r");
+    g_free (command);
+
+    if (fp == NULL)
+        return FALSE;
 
     if (fscanf(fp, "%d", &num_processes) != 1)
             num_processes = 0;
 
     pclose(fp);
-    g_free (command);
 
     if (num_processes > 0) {
         return TRUE;
@@ -251,14 +255,17 @@ do_user_switch (GSLockPlug *plug) {
                                               NULL,
                                               &error);
         if (proxy != NULL) {
-            g_dbus_proxy_call_sync (proxy,
-                                    "SwitchToGreeter",
-                                    g_variant_new ("()"),
-                                    G_DBUS_CALL_FLAGS_NONE,
-                                    -1,
-                                    NULL,
-                                    NULL);
+            GVariant *variant = g_dbus_proxy_call_sync (proxy,
+                                                        "SwitchToGreeter",
+                                                        g_variant_new ("()"),
+                                                        G_DBUS_CALL_FLAGS_NONE,
+                                                        -1,
+                                                        NULL,
+                                                        NULL);
             g_object_unref (proxy);
+            if (variant != NULL) {
+                g_variant_unref (variant);
+            }
         } else {
             gs_debug ("Unable to start LightDM greeter: %s", error->message);
             g_error_free (error);
@@ -269,9 +276,9 @@ do_user_switch (GSLockPlug *plug) {
 static void
 set_status_text (GSLockPlug *plug,
                  const char *text) {
-    if (plug->priv->auth_message_label != NULL) {
+    if (plug->priv->auth_message_label != NULL && g_utf8_validate (text, -1, NULL)) {
         gtk_label_set_text (GTK_LABEL (plug->priv->auth_message_label), text);
-        if (g_utf8_strlen (text, 1) == 0) {
+        if (g_utf8_strlen (text, -1) == 0) {
             gtk_widget_hide (GTK_WIDGET (plug->priv->auth_message_label));
         } else {
             gtk_widget_show (GTK_WIDGET (plug->priv->auth_message_label));
@@ -415,10 +422,7 @@ capslock_update (GSLockPlug *plug,
 }
 
 static gboolean
-is_capslock_on (void) {
-    GdkKeymap *keymap;
-
-    keymap = gdk_keymap_get_for_display (gdk_display_get_default());;
+is_capslock_on (GdkKeymap *keymap) {
     if (keymap == NULL)
         return FALSE;
 
@@ -510,7 +514,7 @@ run_destroy_handler (GSLockPlug *plug,
 static void
 run_keymap_handler (GdkKeymap  *keymap,
                     GSLockPlug *plug) {
-    capslock_update (plug, is_capslock_on ());
+    capslock_update (plug, is_capslock_on (keymap));
 }
 
 /* adapted from GTK+ gtkdialog.c */
@@ -596,7 +600,7 @@ gs_lock_plug_run (GSLockPlug *plug) {
 }
 
 static GdkPixbuf *
-get_user_icon_from_accounts_service (void) {
+get_user_icon_from_accounts_service (gint scale_factor) {
     GDBusConnection *bus;
     GError          *error = NULL;
     GVariant        *variant, *res;
@@ -627,10 +631,10 @@ get_user_icon_from_accounts_service (void) {
         return NULL;
     }
 
-    user_path = g_variant_get_string (g_variant_get_child_value (variant,
-                                                                 0),
-                                      NULL);
+    GVariant *child_val = g_variant_get_child_value (variant, 0);
+    user_path = g_variant_get_string (child_val, NULL);
     g_variant_unref (variant);
+
     variant = g_dbus_connection_call_sync (bus,
                                            "org.freedesktop.Accounts",
                                            user_path,
@@ -642,6 +646,7 @@ get_user_icon_from_accounts_service (void) {
                                            G_VARIANT_TYPE ("(v)"),
                                            G_DBUS_CALL_FLAGS_NONE,
                                            -1, NULL, &error);
+    g_variant_unref (child_val);
     if (variant == NULL) {
         gs_debug ("Could not find user icon: %s", error->message);
         g_error_free (error);
@@ -652,10 +657,12 @@ get_user_icon_from_accounts_service (void) {
     g_variant_get_child (variant, 0, "v", &res);
     pixbuf = gdk_pixbuf_new_from_file_at_scale (g_variant_get_string (res,
                                                                       NULL),
-                                                FACE_ICON_SIZE, FACE_ICON_SIZE, FALSE,
+                                                FACE_ICON_SIZE * scale_factor,
+                                                FACE_ICON_SIZE * scale_factor,
+                                                FALSE,
                                                 &error);
     if (pixbuf == NULL) {
-        g_warning ("Could not load user avatar: %s", error->message);
+        gs_debug ("Could not load user avatar: %s", error->message);
         g_error_free (error);
     }
 
@@ -667,17 +674,18 @@ get_user_icon_from_accounts_service (void) {
 }
 
 static GdkPixbuf *
-logged_in_pixbuf (GdkPixbuf *pixbuf)
+logged_in_pixbuf (GdkPixbuf *pixbuf, gint scale_factor)
 {
     GdkPixbuf *composite = NULL, *emblem = NULL;
-    gint width, height;
+    gint width, height, emblem_width, emblem_height;
     GError *error = NULL;
 
-    emblem = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-                                       LOGGED_IN_EMBLEM_ICON,
-                                       LOGGED_IN_EMBLEM_SIZE,
-                                       GTK_ICON_LOOKUP_FORCE_SIZE,
-                                       &error);
+    emblem = gtk_icon_theme_load_icon_for_scale (gtk_icon_theme_get_default (),
+                                                 LOGGED_IN_EMBLEM_ICON,
+                                                 LOGGED_IN_EMBLEM_SIZE,
+                                                 scale_factor,
+                                                 GTK_ICON_LOOKUP_FORCE_SIZE,
+                                                 &error);
 
     if (!emblem) {
         g_warning ("Failed to load the logged icon: %s", error->message);
@@ -690,13 +698,17 @@ logged_in_pixbuf (GdkPixbuf *pixbuf)
     width = gdk_pixbuf_get_width (composite);
     height = gdk_pixbuf_get_height (composite);
 
+    // Or LOGGED_IN_EMBLEM_SIZE * scale_factor.
+    emblem_width = gdk_pixbuf_get_width (emblem);
+    emblem_height = gdk_pixbuf_get_height (emblem);
+
     gdk_pixbuf_composite (emblem, composite,
-                          width - LOGGED_IN_EMBLEM_SIZE,
-                          height - LOGGED_IN_EMBLEM_SIZE,
-                          LOGGED_IN_EMBLEM_SIZE,
-                          LOGGED_IN_EMBLEM_SIZE,
-                          width - LOGGED_IN_EMBLEM_SIZE,
-                          height - LOGGED_IN_EMBLEM_SIZE,
+                          width - emblem_width,
+                          height - emblem_height,
+                          emblem_width,
+                          emblem_height,
+                          width - emblem_width,
+                          height - emblem_height,
                           1.0,
                           1.0,
                           GDK_INTERP_BILINEAR,
@@ -736,16 +748,25 @@ round_image (GdkPixbuf *pixbuf)
 
 static gboolean
 set_face_image (GSLockPlug *plug) {
-    char      *path;
-    GError    *error = NULL;
-    GdkPixbuf *pixbuf, *temp_image;
+    char            *path;
+    GError          *error = NULL;
+    GdkPixbuf       *pixbuf, *temp_image;
+    cairo_surface_t *surface;
+    gint            scale_factor;
 
-    pixbuf = get_user_icon_from_accounts_service ();
+    scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (plug));
+    pixbuf = get_user_icon_from_accounts_service (scale_factor);
     if (pixbuf == NULL) {
         path = g_build_filename (g_get_home_dir(), ".face", NULL);
-        pixbuf = gdk_pixbuf_new_from_file_at_scale (path, FACE_ICON_SIZE, FACE_ICON_SIZE, FALSE, &error);
+        pixbuf = gdk_pixbuf_new_from_file_at_scale (path,
+                                                    FACE_ICON_SIZE * scale_factor,
+                                                    FACE_ICON_SIZE * scale_factor,
+                                                    FALSE,
+                                                    &error);
+        g_free (path);
+
         if (pixbuf == NULL) {
-            g_warning ("Could not load the user avatar: %s", error->message);
+            gs_debug ("Could not load user avatar: %s", error->message);
             g_error_free (error);
             return FALSE;
         }
@@ -757,13 +778,15 @@ set_face_image (GSLockPlug *plug) {
         pixbuf = temp_image;
     }
 
-    temp_image = logged_in_pixbuf (pixbuf);
+    temp_image = logged_in_pixbuf (pixbuf, scale_factor);
     if (temp_image != NULL) {
         g_object_unref (pixbuf);
         pixbuf = temp_image;
     }
 
-    gtk_image_set_from_pixbuf (GTK_IMAGE (plug->priv->auth_face_image), pixbuf);
+    surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale_factor, NULL);
+    gtk_image_set_from_surface (GTK_IMAGE (plug->priv->auth_face_image), surface);
+    cairo_surface_destroy (surface);
     g_object_unref (pixbuf);
 
     return TRUE;
@@ -772,6 +795,7 @@ set_face_image (GSLockPlug *plug) {
 static void
 gs_lock_plug_show (GtkWidget *widget) {
     GSLockPlug *plug = GS_LOCK_PLUG (widget);
+    GdkKeymap *keymap;
 
     gs_profile_start (NULL);
 
@@ -786,7 +810,9 @@ gs_lock_plug_show (GtkWidget *widget) {
         set_face_image (plug);
     }
 
-    capslock_update (plug, is_capslock_on ());
+    keymap = gdk_keymap_get_for_display (gtk_widget_get_display (GTK_WIDGET (plug)));
+
+    capslock_update (plug, is_capslock_on (keymap));
 
     restart_cancel_timeout (plug);
 
@@ -1013,6 +1039,7 @@ gs_lock_plug_class_init (GSLockPlugClass *klass) {
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
     GtkBindingSet  *binding_set;
 
+    object_class->constructed  = gs_lock_plug_constructed;
     object_class->finalize     = gs_lock_plug_finalize;
     object_class->get_property = gs_lock_plug_get_property;
     object_class->set_property = gs_lock_plug_set_property;
@@ -1045,28 +1072,28 @@ gs_lock_plug_class_init (GSLockPlugClass *klass) {
                                                            NULL,
                                                            NULL,
                                                            FALSE,
-                                                           G_PARAM_READWRITE));
+                                                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
     g_object_class_install_property (object_class,
                                      PROP_LOGOUT_COMMAND,
                                      g_param_spec_string ("logout-command",
                                                           NULL,
                                                           NULL,
                                                           NULL,
-                                                          G_PARAM_READWRITE));
+                                                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
     g_object_class_install_property (object_class,
                                      PROP_STATUS_MESSAGE,
                                      g_param_spec_string ("status-message",
                                                           NULL,
                                                           NULL,
                                                           NULL,
-                                                          G_PARAM_READWRITE));
+                                                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
     g_object_class_install_property (object_class,
                                      PROP_SWITCH_ENABLED,
                                      g_param_spec_boolean ("switch-enabled",
                                                            NULL,
                                                            NULL,
                                                            FALSE,
-                                                           G_PARAM_READWRITE));
+                                                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
     g_object_class_install_property (object_class,
                                      PROP_MONITOR_INDEX,
                                      g_param_spec_int ("monitor-index",
@@ -1075,7 +1102,7 @@ gs_lock_plug_class_init (GSLockPlugClass *klass) {
                                                        0,
                                                        200,
                                                        0,
-                                                       G_PARAM_READWRITE));
+                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
     binding_set = gtk_binding_set_by_class (klass);
 
@@ -1479,15 +1506,20 @@ get_draw_dimensions(GSLockPlug *plug,
 
 static void
 redraw_background (GSLockPlug *plug) {
-    XfceBG    *bg;
-    GdkPixbuf *pixbuf;
-    gint       screen_width, screen_height, monitor_width, monitor_height, scale;
+    cairo_surface_t *surface;
+    XfceBG          *bg;
+    GdkPixbuf       *pixbuf;
+    gint            screen_width, screen_height, monitor_width, monitor_height, scale;
 
     gs_debug ("Redrawing background");
     bg = xfce_bg_new();
     get_draw_dimensions(plug, bg, &screen_width, &screen_height, &monitor_width, &monitor_height, &scale);
     pixbuf = xfce_bg_get_pixbuf(bg, screen_width, screen_height, monitor_width, monitor_height, scale);
-    gtk_image_set_from_pixbuf(GTK_IMAGE(plug->priv->background_image), pixbuf);
+    surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale, NULL);
+    gtk_image_set_from_surface (GTK_IMAGE (plug->priv->background_image), surface);
+    cairo_surface_destroy (surface);
+    g_object_unref (pixbuf);
+    g_object_unref (bg);
 }
 
 static void
@@ -1514,6 +1546,7 @@ gs_lock_plug_add_login_window (GSLockPlug *plug) {
                                       xfce4_screensaver_dialog_ui_length, &error)) {
         g_warning ("Error loading UI: %s", error->message);
         g_error_free(error);
+        g_object_unref (builder);
         return FALSE;
     }
 
@@ -1573,11 +1606,10 @@ gs_lock_plug_add_login_window (GSLockPlug *plug) {
         gtk_widget_set_no_show_all (plug->priv->auth_switch_button, TRUE);
     }
 
-    redraw_background (plug);
-
     date_time_update (plug);
     gtk_widget_show_all (lock_dialog);
 
+    g_object_unref (builder);
     return TRUE;
 }
 
@@ -1692,6 +1724,15 @@ gs_lock_plug_init (GSLockPlug *plug) {
     gs_profile_end (NULL);
 }
 
+static void
+gs_lock_plug_constructed (GObject *object) {
+    GSLockPlug *plug = GS_LOCK_PLUG (object);
+
+    redraw_background (plug);
+
+    G_OBJECT_CLASS (gs_lock_plug_parent_class)->constructed (object);
+}
+
 static void
 gs_lock_plug_finalize (GObject *object) {
     GSLockPlug *plug;
@@ -1716,12 +1757,20 @@ gs_lock_plug_finalize (GObject *object) {
 }
 
 GtkWidget *
-gs_lock_plug_new (void) {
-    GtkWidget *result;
+gs_lock_plug_new (gboolean logout_enabled,
+                  const gchar *logout_command,
+                  gboolean switch_enabled,
+                  const gchar *status_message,
+                  gint monitor_index) {
+    GtkWidget *plug = g_object_new (GS_TYPE_LOCK_PLUG,
+                                    "logout-enabled", logout_enabled,
+                                    "logout-command", logout_command,
+                                    "switch-enabled", switch_enabled,
+                                    "status-message", status_message,
+                                    "monitor-index", monitor_index,
+                                    NULL);
 
-    result = g_object_new (GS_TYPE_LOCK_PLUG, NULL);
+    gtk_window_set_focus_on_map (GTK_WINDOW (plug), TRUE);
 
-    gtk_window_set_focus_on_map (GTK_WINDOW (result), TRUE);
-
-    return result;
+    return plug;
 }
