@@ -30,18 +30,17 @@
 #include <glib-object.h>
 
 #include <xfconf/xfconf.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include "gs-prefs.h"
 
-static void gs_prefs_class_init (GSPrefsClass *klass);
-static void gs_prefs_init       (GSPrefs      *prefs);
 static void gs_prefs_finalize   (GObject      *object);
 
-static GSPrefs *prefs;
+static GSPrefs *global_prefs;
 
 struct GSPrefsPrivate {
     XfconfChannel *channel;
-    XfconfChannel *xfpm_channel;
+    XfceScreensaver *screensaver;
 };
 
 enum {
@@ -200,8 +199,7 @@ _gs_prefs_set_themes (GSPrefs  *prefs,
                       gchar   **values) {
     guint i;
     if (prefs->themes) {
-        g_slist_foreach (prefs->themes, (GFunc)g_free, NULL);
-        g_slist_free (prefs->themes);
+        g_slist_free_full (prefs->themes, g_free);
         prefs->themes = NULL;
     }
 
@@ -333,7 +331,7 @@ gs_prefs_load_from_settings (GSPrefs *prefs) {
                                       DEFAULT_KEY_IDLE_ACTIVATION_ENABLED);
     _gs_prefs_set_idle_activation_enabled (prefs, bvalue);
 
-    bvalue = xfconf_channel_get_bool (prefs->priv->xfpm_channel,
+    bvalue = xfconf_channel_get_bool (prefs->priv->channel,
                                       KEY_LOCK_ON_SLEEP,
                                       DEFAULT_KEY_LOCK_ON_SLEEP);
     _gs_prefs_set_sleep_activation_enabled (prefs, bvalue);
@@ -585,11 +583,8 @@ gs_prefs_init (GSPrefs *prefs) {
                       G_CALLBACK (key_changed_cb),
                       prefs);
 
-    prefs->priv->xfpm_channel = xfconf_channel_get (XFPM_XFCONF_CHANNEL);
-    g_signal_connect (prefs->priv->xfpm_channel,
-                      "property-changed",
-                      G_CALLBACK (key_changed_cb),
-                      prefs);
+    /* keep "/lock/sleep-activation" in sync with other components */
+    prefs->priv->screensaver = xfce_screensaver_new ();
 
     prefs->saver_enabled            = TRUE;
     prefs->lock_enabled             = TRUE;
@@ -621,35 +616,27 @@ gs_prefs_finalize (GObject *object) {
 
     g_return_if_fail (prefs->priv != NULL);
 
-    if (prefs->priv->channel) {
-        g_object_unref (prefs->priv->channel);
-        prefs->priv->channel = NULL;
-    }
-
-    if (prefs->priv->xfpm_channel) {
-        g_object_unref (prefs->priv->xfpm_channel);
-        prefs->priv->xfpm_channel = NULL;
-    }
-
     if (prefs->themes) {
-        g_slist_foreach (prefs->themes, (GFunc)g_free, NULL);
-        g_slist_free (prefs->themes);
+        g_slist_free_full (prefs->themes, g_free);
     }
 
     g_free (prefs->logout_command);
     g_free (prefs->keyboard_command);
 
+    g_signal_handlers_disconnect_by_func (prefs->priv->channel, key_changed_cb, prefs);
+    g_object_unref (prefs->priv->screensaver);
+
     G_OBJECT_CLASS (gs_prefs_parent_class)->finalize (object);
 }
 
 GSPrefs *
 gs_prefs_new (void) {
     GObject *object;
-    if (!prefs) {
+    if (!global_prefs) {
         object = g_object_new (GS_TYPE_PREFS, NULL);
-        prefs = GS_PREFS (object);
+        global_prefs = GS_PREFS (object);
     } else
-       object = g_object_ref (G_OBJECT (prefs));
+       object = g_object_ref (G_OBJECT (global_prefs));
 
     return GS_PREFS (object);
 }
