@@ -81,6 +81,7 @@ struct GSWindowPrivate {
     GtkWidget       *info_content;
 
     guint            popup_dialog_idle_id;
+    guint            deactivated_idle_id;
 
     guint            dialog_map_signal_id;
     guint            dialog_unmap_signal_id;
@@ -426,13 +427,17 @@ emit_deactivated_idle (gpointer user_data) {
     GSWindow *window = user_data;
 
     g_signal_emit (window, signals[DEACTIVATED], 0);
+    window->priv->deactivated_idle_id = 0;
 
     return FALSE;
 }
 
 static void
 add_emit_deactivated_idle (GSWindow *window) {
-    g_idle_add (emit_deactivated_idle, window);
+    if (window->priv->deactivated_idle_id > 0) {
+        g_source_remove (window->priv->deactivated_idle_id);
+    }
+    window->priv->deactivated_idle_id = g_idle_add (emit_deactivated_idle, window);
 }
 
 static void
@@ -1518,6 +1523,7 @@ gs_window_set_monitor (GSWindow   *window,
     }
 
     window->priv->monitor = monitor;
+    g_object_add_weak_pointer (G_OBJECT (monitor), (gpointer *) &window->priv->monitor);
 
     gtk_widget_queue_resize (GTK_WIDGET (window));
     gtk_widget_get_preferred_width (GTK_WIDGET (window), &tmp, &tmp);
@@ -1718,7 +1724,7 @@ gs_window_real_size_request (GtkWidget      *widget,
     child = gtk_bin_get_child (bin);
 
     // monitor been removed, but we will return it later on
-    if (!GDK_IS_MONITOR (window->priv->monitor))
+    if (window->priv->monitor == NULL)
         return;
 
     if (child && gtk_widget_get_visible (child)) {
@@ -1998,6 +2004,13 @@ gs_window_finalize (GObject *object) {
 
     remove_watchdog_timer (window);
     remove_popup_dialog_idle (window);
+    if (window->priv->deactivated_idle_id > 0) {
+        g_source_remove (window->priv->deactivated_idle_id);
+        window->priv->deactivated_idle_id = 0;
+    }
+    if (window->priv->monitor != NULL) {
+        g_object_remove_weak_pointer (G_OBJECT (window->priv->monitor), (gpointer *) &window->priv->monitor);
+    }
 
     if (window->priv->timer) {
         g_timer_destroy (window->priv->timer);
@@ -2008,6 +2021,7 @@ gs_window_finalize (GObject *object) {
     remove_command_watches (window);
 
     gs_window_dialog_finish (window);
+    g_object_unref (window->priv->prefs);
 
     G_OBJECT_CLASS (gs_window_parent_class)->finalize (object);
 }
