@@ -53,7 +53,10 @@ Authors: Soren Sandmann <sandmann@redhat.com>
 #include <xfconf/xfconf.h>
 
 #define XFCE_BG_CACHE_DIR "xfce/background"
-#define XFCE_BG_FALLBACK_IMG "xfce-teal.jpg"
+/* Default wallpaper in Xfce 4.20 */
+#define XFCE_BG_FALLBACK_IMG "xfce-x.svg"
+/* For legacy xfdesktop, this is no longer shipped in 4.20 */
+#define XFCE_BG_FALLBACK_IMG_4_18 "xfce-teal.jpg"
 
 /* We keep the large pixbufs around if the next update
    in the slideshow is less than 60 seconds away */
@@ -96,7 +99,6 @@ struct _XfceBG {
     XfceBGColorType  color_type;
     GdkRGBA          primary;
     GdkRGBA          secondary;
-    gboolean         is_enabled;
 
     GFileMonitor    *file_monitor;
 
@@ -286,19 +288,23 @@ queue_transitioned (XfceBG *bg) {
 }
 
 static gchar *
-find_system_backgrounds (void) {
+find_system_default_background (void) {
     const gchar * const *dirs;
     gint                 i;
 
     dirs = g_get_system_data_dirs ();
     for (i = 0; dirs[i]; i++) {
         gchar *path;
-        path = g_build_path (G_DIR_SEPARATOR_S, dirs[i],
-                             "backgrounds", "xfce", NULL);
-        if (g_file_test (path, G_FILE_TEST_IS_DIR))
+
+        path = g_build_filename (dirs[i], "backgrounds", "xfce", XFCE_BG_FALLBACK_IMG, NULL);
+        if (g_file_test (path, G_FILE_TEST_EXISTS))
             return path;
-        else
-            g_free (path);
+        g_free (path);
+
+        path = g_build_filename (dirs[i], "backgrounds", "xfce", XFCE_BG_FALLBACK_IMG_4_18, NULL);
+        if (g_file_test (path, G_FILE_TEST_EXISTS))
+            return path;
+        g_free (path);
     }
 
     return NULL;
@@ -427,8 +433,7 @@ xfce_bg_load_from_xfconf (XfceBG        *bg,
     prop_prefix = xfce_bg_get_property_prefix (channel, monitor_name);
 
     property = g_strconcat(prop_prefix, "/image-style", NULL);
-    placement = xfconf_channel_get_int(channel, property, XFCE_BG_PLACEMENT_NONE);
-    bg->is_enabled = placement != XFCE_BG_PLACEMENT_NONE;
+    placement = xfconf_channel_get_int(channel, property, XFCE_BG_PLACEMENT_ZOOMED);
 
     /* Filename */
     g_free(property);
@@ -438,10 +443,11 @@ xfce_bg_load_from_xfconf (XfceBG        *bg,
         property = g_strconcat(prop_prefix, "/image-path", NULL);
     }
     filename = NULL;
-    tmp = xfconf_channel_get_string(channel, property,
-                                    g_build_filename(find_system_backgrounds(),
-                                                     XFCE_BG_FALLBACK_IMG,
-                                                     NULL));
+
+    gchar *background_path = find_system_default_background ();
+    tmp = xfconf_channel_get_string (channel, property, background_path);
+    g_free (background_path);
+
     if (tmp && *tmp != '\0') {
         /* FIXME: UTF-8 checks should go away.
          * picture-filename is of type string, which can only be used for
@@ -457,16 +463,10 @@ xfce_bg_load_from_xfconf (XfceBG        *bg,
             filename = g_filename_from_utf8 (tmp, -1, NULL, NULL, NULL);
         }
 
-        /* Fallback to default BG if the filename set is non-existent */
+        /* Fallback to plain color if the filename set is non-existent */
         if (filename != NULL && !g_file_test (filename, G_FILE_TEST_EXISTS)) {
             g_free (filename);
-            filename = g_strdup(XFCE_BG_FALLBACK_IMG);
-
-            //* Check if default background exists, also */
-            if (filename != NULL && !g_file_test (filename, G_FILE_TEST_EXISTS)) {
-                g_free (filename);
-                filename = NULL;
-            }
+            filename = NULL;
         }
     }
     g_free (tmp);
@@ -505,6 +505,7 @@ xfce_bg_load_from_xfconf (XfceBG        *bg,
     ctype = xfconf_channel_get_int(channel, property, XFCE_BG_COLOR_SOLID);
 
     g_free(property);
+    g_free(prop_prefix);
 
     xfce_bg_set_color (bg, ctype, &c1, &c2);
     xfce_bg_set_placement (bg, placement);
@@ -1019,7 +1020,7 @@ xfce_bg_get_pixbuf(XfceBG *bg,
     }
 
     pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8,
-                            width, height);
+                            width * scale, height * scale);
 
     xfce_bg_draw(bg, pixbuf);
 
@@ -1496,22 +1497,28 @@ get_pixbuf_for_size (XfceBG *bg,
                 if (slide->fixed) {
                     FileSize *size = find_best_size (slide->file1,
                                      best_width, best_height);
-                    bg->pixbuf_cache =
-                        get_as_pixbuf_for_size (bg, size->file, monitor,
-                                    best_width, best_height);
+                    if (size != NULL) {
+                        bg->pixbuf_cache =
+                            get_as_pixbuf_for_size (bg, size->file, monitor,
+                                        best_width, best_height);
+                    }
                 } else {
                     FileSize *size;
-                    GdkPixbuf *p1, *p2;
+                    GdkPixbuf *p1 = NULL, *p2 = NULL;
 
                     size = find_best_size (slide->file1,
                                 best_width, best_height);
-                    p1 = get_as_pixbuf_for_size (bg, size->file, monitor,
-                                     best_width, best_height);
+                    if (size != NULL) {
+                        p1 = get_as_pixbuf_for_size (bg, size->file, monitor,
+                                         best_width, best_height);
+                    }
 
                     size = find_best_size (slide->file2,
                                 best_width, best_height);
-                    p2 = get_as_pixbuf_for_size (bg, size->file, monitor,
-                                     best_width, best_height);
+                    if (size != NULL) {
+                        p2 = get_as_pixbuf_for_size (bg, size->file, monitor,
+                                         best_width, best_height);
+                    }
 
                     if (p1 && p2)
                         bg->pixbuf_cache = blend (p1, p2, alpha);
