@@ -48,7 +48,12 @@ main (int    argc,
         } else {
             g_warning ("Unable to initialize GTK+");
         }
-        exit (1);
+        return EXIT_FAILURE;
+    }
+
+    if (!GDK_IS_X11_DISPLAY (gdk_display_get_default ())) {
+        g_warning ("Unsupported windowing environment");
+        return EXIT_FAILURE;
     }
 
     display = gdk_display_get_default ();
@@ -62,5 +67,5 @@ main (int    argc,
         printf ("none\n");
     }
 
-    return 0;
+    return EXIT_SUCCESS;
 }
