@@ -47,7 +47,7 @@
 #include "xfce4-screensaver-preferences-ui.h"
 
 #define GPM_COMMAND "xfce4-power-manager-settings"
-#define CONFIGURE_COMMAND "xfce4-screensaver-configure"
+#define CONFIGURE_COMMAND "xfce4-screensaver-configure.py"
 
 enum {
     NAME_COLUMN = 0,
@@ -767,6 +767,7 @@ tree_selection_previous (GtkTreeSelection *selection) {
     if (gtk_tree_path_prev (path)) {
         gtk_tree_selection_select_path (selection, path);
     }
+    gtk_tree_path_free (path);
 }
 
 static void
@@ -782,6 +783,7 @@ tree_selection_next (GtkTreeSelection *selection) {
     path = gtk_tree_model_get_path (model, &iter);
     gtk_tree_path_next (path);
     gtk_tree_selection_select_path (selection, path);
+    gtk_tree_path_free (path);
 }
 
 static void
@@ -965,6 +967,7 @@ separator_func (GtkTreeModel *model,
     gtk_tree_model_get (model, iter, column, &text, -1);
 
     if (text != NULL && strcmp (text, "__separator") == 0) {
+        g_free (text);
         return TRUE;
     }
 
@@ -2023,9 +2026,14 @@ main (int    argc,
         return EXIT_FAILURE;
     }
 
+    if (!GDK_IS_X11_DISPLAY (gdk_display_get_default ())) {
+        g_warning ("Unsupported windowing environment");
+        return EXIT_FAILURE;
+    }
+
     /* hook to make sure the libxfce4ui library is linked */
     if (xfce_titled_dialog_get_type() == 0)
-        exit(1);
+        return EXIT_FAILURE;
 
     if (!xfconf_init(&error)) {
         g_error("Failed to connect to xfconf daemon: %s.", error->message);
@@ -2093,5 +2101,5 @@ main (int    argc,
 
     xfconf_shutdown ();
 
-    return 0;
+    return EXIT_SUCCESS;
 }
