@@ -32,6 +32,7 @@
 #include <sys/wait.h>
 #include <unistd.h>
 
+#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
 #include <libxfce4util/libxfce4util.h>
@@ -51,23 +52,15 @@ int main(int    argc,
     GError    *error = NULL;
 
     static gboolean show_version = FALSE;
-    static gboolean no_daemon = TRUE;
     static gboolean debug = FALSE;
 
     static GOptionEntry entries[] = {
         {"version", 0, 0, G_OPTION_ARG_NONE, &show_version, N_("Version of this application"), NULL},
-        {"no-daemon", 0, 0, G_OPTION_ARG_NONE, &no_daemon, N_("Don't become a daemon"), NULL},
         {"debug", 0, 0, G_OPTION_ARG_NONE, &debug, N_("Enable debugging code"), NULL},
         {NULL}
     };
 
-    #ifdef ENABLE_NLS
-        bindtextdomain(GETTEXT_PACKAGE, XFCELOCALEDIR);
-        #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
-            bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
-        #endif
-        textdomain(GETTEXT_PACKAGE);
-    #endif
+    xfce_textdomain (GETTEXT_PACKAGE, XFCELOCALEDIR, "UTF-8");
 
     if (!gtk_init_with_args(&argc, &argv, NULL, entries, NULL, &error)) {
         if (error) {
@@ -77,12 +70,17 @@ int main(int    argc,
             g_warning("Unable to initialize GTK+");
         }
 
-        exit(1);
+        return EXIT_FAILURE;
     }
 
     if (show_version) {
         g_print("%s %s\n", argv[0], VERSION);
-        exit(1);
+        return EXIT_SUCCESS;
+    }
+
+    if (!GDK_IS_X11_DISPLAY (gdk_display_get_default ())) {
+        g_warning ("Unsupported windowing environment");
+        return EXIT_FAILURE;
     }
 
     if (!xfconf_init(&error)) {
@@ -99,7 +97,7 @@ int main(int    argc,
     monitor = gs_monitor_new();
 
     if (monitor == NULL) {
-        exit (1);
+        return EXIT_FAILURE;
     }
 
     error = NULL;
@@ -112,7 +110,8 @@ int main(int    argc,
             g_warning("Unable to start screensaver");
         }
 
-        exit(1);
+        g_object_unref(monitor);
+        return EXIT_FAILURE;
     }
 
     gtk_main();
@@ -122,6 +121,7 @@ int main(int    argc,
     gs_debug("xfce4-screensaver finished");
 
     gs_debug_shutdown();
+    xfconf_shutdown ();
 
     return 0;
 }
